<?php

use Hepsipay\HepsiPayGateway;

if ( ! defined( "ALTTANTIRE_MIN_PHP_VER" ) ) {
	define( 'ALTTANTIRE_MIN_PHP_VER', '7.4.0' );
}

if ( ! defined( "HEPSIPAY_VERSION" ) ) {
	define( 'HEPSIPAY_VERSION', '4.0.0' );
}

class ControllerExtensionPaymentHepsipayVpos extends Controller {
	protected HepsiPayGateway $Gateway;
	private array $error = array();
	private array $success = array();

	public function __construct( $registry ) {
		parent::__construct( $registry );

		include_once DIR_SYSTEM . "/library/hp-payment-sdk/Gateway.php";

		if ( $this->config->get( 'hepsipay_vpos_environment' ) &&
		     $this->config->get( 'hepsipay_vpos_merchant_no' ) &&
		     $this->config->get( 'hepsipay_vpos_terminal_no' ) &&
		     $this->config->get( 'hepsipay_vpos_store_key' ) ) {

			$this->Gateway = new HepsiPayGateway( $this->config->get( 'hepsipay_vpos_environment' ),
				$this->config->get( 'hepsipay_vpos_merchant_no' ),
				$this->config->get( 'hepsipay_vpos_terminal_no' ),
				$this->config->get( 'hepsipay_vpos_store_key' ) );
		}

	}

	public function index(): void {
		$this->load->language( 'extension/payment/hepsipay_vpos' );
		$this->load->model( 'setting/setting' );

		if ( $this->request->server['REQUEST_METHOD'] == 'POST' ) {
			$this->save();
		}

		$this->document->setTitle( $this->language->get( 'heading_title' ) );
		$data['heading_title'] = $this->language->get('heading_title');

		////


		$data['text_edit'] = $this->language->get('text_edit');
		$data['text_enabled'] = $this->language->get('text_enabled');
		$data['text_disabled'] = $this->language->get('text_disabled');
		$data['text_active'] = $this->language->get('text_active');
		$data['text_passive'] = $this->language->get('text_passive');
		$data['text_select'] = $this->language->get('text_select');

		$data['button_save'] = $this->language->get('button_save');
		$data['button_cancel'] = $this->language->get('button_cancel');

		$data['title_api_user_information'] = $this->language->get('title_api_user_information');
		$data['title_order_status'] = $this->language->get('title_order_status');
		$data['title_module_settings'] = $this->language->get('title_module_settings');
		$data['title_system_info'] = $this->language->get('title_system_info');

		$data['label_software_versions'] = $this->language->get('label_software_versions');
		$data['label_system'] = $this->language->get('label_system');
		$data['label_environment'] = $this->language->get('label_environment');
		$data['label_hepsipay_vpos_merchant_no'] = $this->language->get('label_hepsipay_vpos_merchant_no');
		$data['label_hepsipay_vpos_terminal_no'] = $this->language->get('label_hepsipay_vpos_terminal_no');
		$data['label_hepsipay_vpos_store_key'] = $this->language->get('label_hepsipay_vpos_store_key');
		$data['label_tl_currency'] = $this->language->get('label_tl_currency');
		$data['label_module_status'] = $this->language->get('label_module_status');
		$data['label_hepsipay_vpos_tl_currency'] = $this->language->get('label_hepsipay_vpos_tl_currency');
		$data['label_order_completed'] = $this->language->get('label_order_completed');





		////
		$data['breadcrumbs'] = [];
		$data['breadcrumbs'][] = [
			'text' => $this->language->get( 'text_home' ),
			'href' => $this->url->link( 'common/dashboard', 'token=' . $this->session->data['token'] )
		];
		$data['breadcrumbs'][] = [
			'text' => $this->language->get( 'text_extension' ),
			'href' => $this->url->link( 'marketplace/extension',
				'token=' . $this->session->data['token'] . '&type=payment' )
		];

		$data['cancel'] = $this->url->link( 'extension/extension',
			'token=' . $this->session->data['token'] . '&type=payment' );

		$data['breadcrumbs'][] = [
			'text' => $this->language->get( 'heading_title' ),
			'href' => $data['cancel']
		];

		$data['action'] = $this->url->link( 'extension/payment/hepsipay_vpos',
			'token=' . $this->session->data['token'] );


		$data['errors_message'] = array(
			'warning'                                  => $this->language->get( 'error_warning' ),
			'hepsipay_vpos_merchant_no'        => $this->language->get( 'error_merchant_no' ),
			'hepsipay_vpos_terminal_no'        => $this->language->get( 'terminal_no' ),
			'hepsipay_vpos_store_key'          => $this->language->get( 'error_store_key' ),
			'hepsipay_vpos_environment'        => $this->language->get( 'error_environment' ),
			'hepsipay_vpos_order_completed_id' => $this->language->get( 'error_order_completed_id' ),
			'hepsipay_vpos_tl_currency'        => $this->language->get( 'error_tl_currency' ),
			'hepsipay_vpos_merchant_general'           => $this->language->get( 'error_hepsipay_vpos_merchant_general' ),
		);

		$items = [
			"hepsipay_vpos_environment",
			"hepsipay_vpos_merchant_no",
			"hepsipay_vpos_terminal_no",
			"hepsipay_vpos_store_key",
			"hepsipay_vpos_order_completed_id",
			"hepsipay_vpos_tl_currency",
		];

		foreach ( $items as $name ) {

			if ( isset( $this->request->post[ $name ] ) ) {
				$data[ $name ] = trim( $this->request->post[ $name ] );
			} else {
				$data[ $name ] = $this->config->get( $name );
			}

		}

		$this->load->model( "localisation/currency" );
		$data['store_currencies'] = $this->model_localisation_currency->getCurrencies();

		$this->load->model( 'localisation/order_status' );
		$data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

		if ( ! $this->config->get( 'hepsipay_vpos_environment' ) ||
		     ! $this->config->get( 'hepsipay_vpos_merchant_no' ) ||
		     ! $this->config->get( 'hepsipay_vpos_terminal_no' ) ||
		     ! $this->config->get( 'hepsipay_vpos_store_key' ) ) {
			$this->error['hepsipay_merchant_general'] = $this->language->get( 'error_empty_install' );
		} else {
			try {
				if ( $this->check_minimum_requirements() ) {
					$this->success[] = $this->language->get( 'success_install' );
				}
			} catch ( Exception $e ) {
				$this->error[] = "<strong>" . $e->getMessage() . "</strong> " . $this->language->get( 'error_fail_install' );
			}
		}

		$this->load->model( 'extension/payment/hepsipay_vpos' );

		$data['errors'] = $this->error;
		$data['success'] = $this->success;
		$data['hepsipay_vpos_status'] = $this->config->get( 'hepsipay_vpos_status' );
		$data['hepsipay_vpos_sort_order'] = $this->config->get( 'hepsipay_vpos_sort_order' );
		$data['header'] = $this->load->controller( 'common/header' );
		$data['column_left'] = $this->load->controller( 'common/column_left' );
		$data['footer'] = $this->load->controller( 'common/footer' );
		$data['system_info'] = [
			"opencart"          => defined( 'VERSION' ) ? VERSION : "N/A",
			"hepsipay_plugin"   => defined( 'HEPSIPAY_VERSION' ) ? HEPSIPAY_VERSION : "N/A",
			"php_compatibility" => version_compare( phpversion(), ALTTANTIRE_MIN_PHP_VER, '>=' ),
			"php_version"       => phpversion(),
			"curl"              => function_exists( 'curl_init' ),
			"file_get_contents" => function_exists( 'file_get_contents' ) && ini_get( 'allow_url_fopen' ),
		];

//		print_r($data);

		$this->response->setOutput( $this->load->view( 'extension/payment/hepsipay_vpos', $data ) );

	}

	private function save(): void {

		$this->request->post['hepsipay_vpos_merchant_no'] = intval(
			$this->request->post['hepsipay_vpos_merchant_no']
		);

		if ( $this->user->hasPermission( 'modify', 'extension/payment/hepsipay_vpos' ) && $this->validate() ) {
			$this->model_setting_setting->editSetting( 'hepsipay_vpos', $this->request->post );
			$this->success[] = 'Sanal POS ayarları kaydedildi';
		} else {
			$this->error[] = "Kullanıcı bilgileri hatalı!";
		}

	}

	protected function validate(): bool {

		$this->load->model( 'extension/payment/hepsipay_vpos' );

		$error_msg = null;

		if ( ! $this->request->post['hepsipay_vpos_environment'] ) {
			$error_msg = $this->language->get( 'error_environment' );
		}
		if ( ! $this->request->post['hepsipay_vpos_merchant_no'] ) {
			$error_msg = $this->language->get( 'error_merchant_no' );
		}
		if ( ! $this->request->post['hepsipay_vpos_terminal_no'] ) {
			$error_msg = $this->language->get( 'terminal_no' );
		}
		if ( ! $this->request->post['hepsipay_vpos_store_key'] ) {
			$error_msg = $this->language->get( 'error_store_key' );
		}
		if ( ! $this->request->post['hepsipay_vpos_order_completed_id'] ) {
			$error_msg = $this->language->get( 'error_order_completed_id' );
		}
		if ( ! $this->request->post['hepsipay_vpos_tl_currency'] ) {
			$error_msg = $this->language->get( 'error_tl_currency' );
		}

		if ( $error_msg ) {
			return $error_msg;
		}

		$check_response = $this->model_extension_payment_hepsipay_vpos->checkCredential( $this->request->post );
		if ( $check_response === true ) {
			return true;
		}

		$this->error[] = $check_response;

		return false;

	}

	public function install(): void {
		if ( $this->user->hasPermission( 'modify', 'extension/payment' ) ) {
			$this->load->model( 'extension/payment/hepsipay_vpos' );
			$this->model_extension_payment_hepsipay_vpos->install();
		}
	}

	public function uninstall(): void {
		if ( $this->user->hasPermission( 'modify', 'extension/payment' ) ) {
			$this->load->model( 'extension/payment/hepsipay_vpos' );
			$this->model_extension_payment_hepsipay_vpos->uninstall();
		}
	}

	private function check_minimum_requirements(): bool {
		if ( version_compare( phpversion(), ALTTANTIRE_MIN_PHP_VER, '<' ) ) {
			$this->error[] = "Web sitenizde PHP " . phpversion() .
			                 " Versiyonunu kullanıyorsunuz. Hepsipay Sanal POS eklentisi, PHP " .
			                 ALTTANTIRE_MIN_PHP_VER .
			                 " ve üzeri versiyonda çalışır.";

			return false;
		}
		if ( ! function_exists( 'curl_init' ) &&
		     ! function_exists( 'file_get_contents' ) &&
		     ! ini_get( 'allow_url_fopen' ) ) {
			$this->error[] = "Sunucunuzda cURL kurulu ya da aktif değil. " .
			                 "Hepsipay Sanal POS'u kullanabilmeniz için sunucunuzda cURL kurulu olmalıdır.";

			return false;
		}

		return true;
	}
}

