<?php
/**
 * Copyright (c) 2024
 * @author: Alttantire Yazılım Çözümleri <info@alttantire.com>
 * Web: https://www.alttantire.com
 *
 */

use Hepsipay\HepsiPayGateway;

class ModelExtensionPaymentHepsipayVpos extends Model {
	public function checkCredential( $posted ) {
		$gateway = new HepsiPayGateway(
			$posted['hepsipay_vpos_environment'],
			$posted['hepsipay_vpos_merchant_no'],
			$posted['hepsipay_vpos_terminal_no'],
			$posted['hepsipay_vpos_store_key']
		);

		$gateway->set_amount( 100 );
		$gateway->set_account_key( "905551234567" );

		$gateway->set_merchant_callback_url(
			$this->url->link( 'extension/payment/hepsipay_vpos' )
		);

		$gateway->set_merchant_order_number( 1 );
		$gateway->set_is_guest_user( true );
		$gateway->set_force_3ds( true );
		$gateway->set_allowed_installments( [ 1 ] );
		$gateway->set_frame_type( 1 );
		$gateway->set_basket_item( [
			"Product"                => "Credential Check",
			"Price"                  => 100,
			"SubMerchantMemberId"    => null,
			"SubMerchantMemberPrice" => null,
			"CategoryId"             => null,
			"ExternalId"             => null,
		] );

		try {
			// Check Availability
			$frame_check_availability_response = $gateway->check_availability();

			if ( $frame_check_availability_response->Success === true ) {
				$frame_init_response = $gateway->frame_init();
				if ( $frame_init_response && $frame_init_response->Success === true ) {
					return true;
				} else {
					throw new InvalidArgumentException( $frame_init_response->Message );
				}
			} else {
				throw new InvalidArgumentException( $frame_check_availability_response->Message );
			}
		} catch ( Exception $e ) {
			return $e->getMessage();
		}
	}

	public function get_extension_version(): string {
		$json_file = DIR_APPLICATION . '../extension/hepsipay/install.json';

		if ( file_exists( $json_file ) ) {
			$json_content = file_get_contents( $json_file );
			$json_data = json_decode( $json_content, true );
			if ( isset( $json_data['version'] ) ) {
				return $json_data['version'];
			} else {
				return 'N/A';
			}
		} else {
			return 'N/A';
		}
	}

	public function install(): void {
		//install
	}

	public function uninstall(): void {
		$this->load->model( 'setting/setting' );

		$blank_data = array(
			"hepsipay_vpos_environment"        => "",
			"hepsipay_vpos_merchant_no"        => "",
			"hepsipay_vpos_terminal_no"        => "",
			"hepsipay_vpos_store_key"          => "",
			"hepsipay_vpos_order_completed_id" => ""
		);

		$this->model_setting_setting->editSetting( 'hepsipay', $blank_data );

	}

}
